@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
 ;;; Copyright © 2018, 2019 Henk Katerberg <hank@mudball.nl>
-;;; Copyright © 2018, 2019, 2020, 2021 Paul Hoogendijk <paul@dezyne.org>
+;;; Copyright © 2018, 2019, 2020, 2021, 2022 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2019, 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2021 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
@@ -651,16 +651,25 @@ required to be non-deterministic."
   (label transition-label)
   (to transition-to set-transition-to!))
 
-(define* (generate-trace root nodes provides-in fout out #:key verbose?)
+(define* (generate-trace root nodes provides-ports provides-in fout out #:key verbose?)
 
   (define (allowed-end? node)
     (define (label-provides-in? label)
       (and (pair? provides-in)
            (member label provides-in)))
-    (let ((edges (node-succ node)))
-      (or (equal? (node-state node) (1- (vector-length nodes)))
-          (and (find (lambda (e) (label-provides-in? (edge-label e))) edges)
-               (not (find (lambda (e) (equal? "<ack>" (edge-label e))) edges))))))
+    (define (get-port label)
+      (car (string-split label #\.)))
+    (define (idle-node? node)
+      (let* ((labels (map edge-label (node-succ node)))
+             (labels (filter label-provides-in? labels))
+             (ports (map get-port labels))
+             (ports (delete-duplicates ports string=?)))
+       (= (length ports)
+          (length provides-ports))))
+    (or (equal? (node-state node) (1- (vector-length nodes)))
+        (and (idle-node? node)
+             (not (find (lambda (e) (equal? "<ack>" (edge-label e)))
+                        (node-succ node))))))
 
   (define (annotate)
     (let* ((frontier (map node-state
@@ -767,8 +776,8 @@ required to be non-deterministic."
            (nodes (lts->nodes lts #t))
            (nodes (remove-state-edges nodes))
            (root (car (lts-state lts))))
-      (generate-trace root nodes provides-in (string-append model ".trace") out
-                      #:verbose? verbose?))))
+      (generate-trace root nodes provides-ports provides-in
+                      (string-append model ".trace") out #:verbose? verbose?))))
 
 
 ;;;
@@ -777,7 +786,7 @@ required to be non-deterministic."
 
 (define (parse-label label)
   (define-peg-string-patterns
-    "tree               <-- event / modeling / reply / state / return / queue / tau-literal / illegal / error / end / flush / parse-error
+    "tree               <-- event / modeling / reply / state / return / queue / tau-literal / illegal / error / end / flush / blocking / parse-error
      parse-error        <-- [a-zA-Z_0-9'()]*
      event              <-- port-name tick direction lpar scope* action-literal lpar scope* direction tick event-name rpar rpar
      modeling           <-- port-name tick internal-literal lpar scope* ('inevitable' / 'optional') rpar
@@ -785,14 +794,15 @@ required to be non-deterministic."
      end                <   scope* ('end' / 'silent_end')
      return             <-- 'return'
      flush              <-- port-scope* identifier tick 'flush'
+     blocking           <-- port-scope* identifier tick 'blocking'
      reply              <-- port-name tick reply-literal lpar scope* reply-value rpar
      state              <-- port-name tick state-literal state-arguments
      state-literal      <   'state'
      state-arguments    <-  (lpar state-argument (comma state-argument)* rpar)?
      state-argument     <-- bool / int / enum-literal
      scope              <   identifier tick
-     port-scope         <   scope !(internal-literal / queue-direction / direction / reply-literal / state-literal / 'queue_full' / 'flush')
      port-name          <-  port-scope* identifier
+     port-scope         <   scope !(internal-literal / queue-direction / direction / reply-literal / state-literal / 'queue_full' / 'flush' / 'blocking')
      event-name         <-  identifier
      reply-value        <-  bool-literal lpar bool rpar / lpar enum-literal rpar / int-literal lpar int rpar / void-literal lpar void rpar
      bool-literal       <   'Bool'
@@ -851,6 +861,7 @@ required to be non-deterministic."
       (('error ('identifier port) error) (cleanup-error error))
       (('event ('identifier port) ('identifier event)) (string-append port "." event))
       (('flush ('identifier port) "flush") (string-append port ".<flush>"))
+      (('blocking ('identifier port) "blocking") (string-append port ".<blocking>"))
       (('illegal illegal) (and illegal? (cleanup-error illegal)))
       (('modeling ('identifier port) event) (and internal? (string-append port "." event)))
       (('queue ('identifier port) ('queue-direction direction) ('identifier event)) (and internal? (string-append port "." direction "." event)))
