@@ -74,14 +74,10 @@
 ;; Are we running "explore"?
 (define %exploring? (make-parameter #f))
 
-
 ;;;
-;;; Run
+;;; Trace utilities
 ;;;
 
-;;; ’run’ loops over ’step’ until ’rtc?’ (run to completion done) and
-;;; collects a trace (of program counters).
-
 (define (trace-valid? trace)
   ((compose not .status car) trace))
 
@@ -348,8 +344,11 @@ program-counters produced by taking a step."
                                      (is-a? o <trigger-return>)))
                     (observable (and observable? (and=> (trace->trail pc) cdr)))
                     (pcs (step pc o))
-                    (new-pc (and (pair? pcs) (car pcs)))
-                    (trace (if (any (cute blocked-collaterally? pc <>) pcs)
+                    (trace (if (any (disjoin
+                                     (conjoin (is-status? <second-reply-error>)
+                                              (const (is-a? (.statement pc)
+                                                            <trigger-return>)))
+                                     (cute blocked-collaterally? pc <>)) pcs)
                                (cdr trace)
                                trace))
                     (input pc (if (and observable (not (interactive?)))
@@ -380,6 +379,14 @@ program-counters produced by taking a step."
                 (else
                  (append-map loop traces)))))))))
 
+
+;;;
+;;; Run
+;;;
+
+;;; ’run’ loops over ’step’ until ’rtc?’ (run to completion done) and
+;;; collects a trace (of program counters).
+
 (define-method (run-to-completion-unmemoized (pc <program-counter>))
   "Return a list of traces produced by taking steps, starting from
 PC until RTC?."
@@ -533,7 +540,7 @@ with EVENT as first step, until RTC?."
 of) TRACE, extending TRACE."
   (let* ((pc (car trace))
          (traces (producer pc)))
-    (map (cut append <> trace) traces)))
+    (map (cute append <> trace) traces)))
 
 (define-method (run-to-completion (trace <list>) event)
   "Return a list of traces produced by RUN-TO-COMPLETION, extending TRACE."
@@ -550,14 +557,18 @@ until RTC?."
   "Return a list of traces produced by RUN-FLUSH, extending TRACE."
   (extend-trace trace run-flush))
 
+(define-method (run-flush (trace <list>) (instance <runtime:component>))
+  (extend-trace trace (cute run-flush <> instance)))
+
 (define-method (run-flush (pc <program-counter>))
   "Return a list of traces produced by taking steps, starting by flushing (%SUT),
 until RTC?."
-  (let* ((pc (flush pc))
-         (traces (run-to-completion-unmemoized pc))
-         (pcs (map car traces)))
-  (if (every q-empty? pcs) traces
-      (append-map run-flush traces))))
+  (let ((pc (flush pc)))
+    (run-to-completion-unmemoized pc)))
+
+(define-method (run-flush (pc <program-counter>) (instance <runtime:component>))
+  (let ((pc (clone pc #:instance instance)))
+    (run-flush pc)))
 
 (define-method (run-silent (pc <program-counter>) (port <runtime:port>))
   (define (update-state pc port-pc)
@@ -569,7 +580,7 @@ until RTC?."
                (traces (parameterize ((%sut port)
                                       (%exploring? #t)
                                       (%strict? #t))
-                         (append-map (cut run-to-completion ipc <>) modeling-names)))
+                         (append-map (cute run-to-completion ipc <>) modeling-names)))
                (traces (filter (conjoin (compose null? trace->trail)
                                         (compose (negate .status) car))
                                traces)))
@@ -606,7 +617,7 @@ until RTC?."
                                             (%liveness? 'component)
                                             (%exploring? #t)
                                             (%strict? #f))
-                               (append-map (cut run-to-completion ipc <>) modeling-names)))
+                               (append-map (cute run-to-completion ipc <>) modeling-names)))
                      (traces (filter (compose
                                       (disjoin not (is? <queue-full-error>))
                                       .status car)
@@ -641,13 +652,13 @@ until RTC?."
              (silent-pcs (map car silent-traces))
              (pcs (cons pc silent-pcs))
              (traces (map list pcs)))
-        (append-map (cut run-to-completion <> event) (cons (list pc) traces))))
+        (append-map (cute run-to-completion <> event) (cons (list pc) traces))))
      (else
       (let* ((modeling? (member event '("inevitable" "optional")))
              (pc (if modeling? pc
                      (clone pc #:trail (cons event (.trail pc)))))
              (modeling-names (if modeling? (list event) (modeling-names)))
-             (traces (append-map (cut run-to-completion pc <>) modeling-names))
+             (traces (append-map (cute run-to-completion pc <>) modeling-names))
              (traces (parameterize ((%modeling? modeling?))
                        (filter event-executed? traces))))
         traces)))))
@@ -673,8 +684,8 @@ until RTC?."
          (modeling-names (modeling-names interface))
          (trail (cons event (.trail pc)))
          (pc (clone pc #:trail trail #:status #f))
-         (modeling-events (map (cut string-append port-name "." <>) modeling-names))
-         (traces (append-map (cut run-to-completion pc <>) modeling-events))
+         (modeling-events (map (cute string-append port-name "." <>) modeling-names))
+         (traces (append-map (cute run-to-completion pc <>) modeling-events))
          (traces (filter (cute event-executed? port-instance <>) traces))
          (errors (filter (compose .status car) traces)))
 
@@ -683,14 +694,11 @@ until RTC?."
                (component-trigger (trigger->component-trigger component-port
                                                               trigger))
                (instance (.container component-port))
-               (traces (map (cut rewrite-trace-head
-                              (cut clone <> #:instance instance) <>)
-                            traces))
                (collateral-blocked? (or (blocked-on-boundary? pc)
                                         (and (get-handling pc instance)
                                              (blocked-port pc instance))))
                (traces (if collateral-blocked? traces
-                           (append-map run-flush traces))))
+                           (append-map (cute run-flush <> instance) traces))))
           traces))))
 
 (define-method (run-async-event (pc <program-counter>))
@@ -707,7 +715,7 @@ until RTC?."
           (else
            (let* ((pc (car trace))
                   (traces (flush-async pc trace)))
-             (map (cut append <> trace) traces))))))
+             (map (cute append <> trace) traces))))))
 
 (define (did-provides-out? trace)
   (let* ((trail (map cdr (trace->trail trace)))
